@@ -23,8 +23,26 @@ def product_overview_feature_div_table(selector: parsel.Selector, field: str) ->
     return s
 
 
+def product_details_table(selector: parsel.Selector, field: str) -> str:
+    table = selector.xpath(
+        "//table[@id='productDetails_detailBullets_sections1']|"
+        "//table[@id='productDetails_techSpec_section_1']"
+    ).get()
+    if table is None:
+        return ""
+    table = pandas.read_html(table)
+    if len(table) <= 0:
+        return ""
+    table = table[0].set_index(0)
+    if field not in table[1]:
+        return ""
+    s = table[1][field]
+    return s
+
+
 def parse(html: str, p: Product):
     sel = parsel.Selector(text=html)
+    parse_product_asin(p, selector=sel)
     parse_parent_asin(p, selector=sel)
     parse_weight(p, selector=sel)
     parse_brand(p, selector=sel)
@@ -36,28 +54,22 @@ def parse(html: str, p: Product):
     parse_material(p, selector=sel)
     parse_description(p, selector=sel)
     parse_title(p, selector=sel)
+    parse_size(p, selector=sel)
+    parse_color(p, selector=sel)
+    parse_variation(p, selector=sel)
 
 
-def parse_parent_asin(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_product_asin(p: Product, selector: parsel.Selector = None):
+    p.product_asin = selector.re_first(r'"productAsin":"(.+?)"')
+
+
+def parse_parent_asin(p: Product, selector: parsel.Selector = None):
     p.parent_asin = selector.re_first(r'"parentAsin":"(.+?)"')
 
 
-def parse_weight(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_weight(p: Product, selector: parsel.Selector = None):
     def s1() -> str:
-        table = selector.xpath(
-            "//table[@id='productDetails_detailBullets_sections1']|"
-            "//table[@id='productDetails_techSpec_section_1']"
-        ).get()
-        if table is None:
-            return ""
-        table = pandas.read_html(table)
-        if len(table) <= 0:
-            return ""
-        table = table[0].set_index(0)
-        if "Item Weight" not in table[1]:
-            return ""
-        s = table[1]["Item Weight"]
-        return s
+        return product_details_table(selector, "Item Weight")
 
     def s2() -> str:
         ul = selector.xpath("//div[@id='detailBullets_feature_div']/ul")
@@ -85,7 +97,7 @@ def parse_weight(p: Product, html: str = None, selector: parsel.Selector = None)
         return
 
 
-def parse_brand(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_brand(p: Product, selector: parsel.Selector = None):
     def s1() -> str:
         return product_overview_feature_div_table(selector, "Brand")
 
@@ -102,7 +114,7 @@ def parse_brand(p: Product, html: str = None, selector: parsel.Selector = None):
     p.brand = brand
 
 
-def parse_price(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_price(p: Product, selector: parsel.Selector = None):
     def table() -> pandas.DataFrame:
         t = pandas.DataFrame(index=("origin", "discount"), columns=('title', 'price'))
         for i, tr in enumerate(selector.xpath("//div[@id='price']/table/tr")[:2]):
@@ -141,7 +153,7 @@ def parse_price(p: Product, html: str = None, selector: parsel.Selector = None):
         p.price_max = float(prices[1])
 
 
-def parse_in_stock(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_in_stock(p: Product, selector: parsel.Selector = None):
     in_stock = selector.xpath("//div[@id='availability']/span/text()").get()
     if in_stock is None:
         p.in_stock = None
@@ -161,7 +173,7 @@ def parse_in_stock(p: Product, html: str = None, selector: parsel.Selector = Non
         p.in_stock = None
 
 
-def parse_stock(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_stock(p: Product, selector: parsel.Selector = None):
     stock = selector.xpath("//div[@id='availability']/span/text()").get()
     if stock is None:
         p.stock = None
@@ -174,7 +186,7 @@ def parse_stock(p: Product, html: str = None, selector: parsel.Selector = None):
     p.stock = parse_number(stock)
 
 
-def parse_ship(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_ship(p: Product, selector: parsel.Selector = None):
     def s1() -> str:
         table = selector.xpath("//table[@id='tabular-buybox-container']").get()
         if table is None:
@@ -208,7 +220,7 @@ def parse_ship(p: Product, html: str = None, selector: parsel.Selector = None):
         p.ship = None
 
 
-def parse_delivery(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_delivery(p: Product, selector: parsel.Selector = None):
     delivery = selector.xpath("string(//div[@id='contextualIngressPtLabel_deliveryShortLine'])").get()
     if delivery is None:
         p.delivery = None
@@ -227,7 +239,7 @@ def parse_delivery(p: Product, html: str = None, selector: parsel.Selector = Non
         p.delivery = delivery.group(1)
 
 
-def parse_material(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_material(p: Product, selector: parsel.Selector = None):
     material = product_overview_feature_div_table(selector, "Material")
     if material == "":
         p.material = None
@@ -235,7 +247,7 @@ def parse_material(p: Product, html: str = None, selector: parsel.Selector = Non
         p.material = material
 
 
-def parse_description(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_description(p: Product, selector: parsel.Selector = None):
     texts = selector.xpath("//div[@id='feature-bullets']//li[not(@id)]/span/text()").getall()
     if len(texts) < 1:
         p.description = None
@@ -247,6 +259,48 @@ def parse_description(p: Product, html: str = None, selector: parsel.Selector =
     p.description = desc
 
 
-def parse_title(p: Product, html: str = None, selector: parsel.Selector = None):
+def parse_title(p: Product, selector: parsel.Selector = None):
     title = selector.xpath("normalize-space(//span[@id='productTitle']/text())").get()
     p.title = title
+
+
+def parse_size(p: Product, selector: parsel.Selector = None):
+    dimensions = product_details_table(selector, "Product Dimensions")
+    dimensions = dimensions or product_details_table(selector, "Package Dimensions")
+    if not dimensions.endswith("inches"):
+        p.size_length = None
+        p.size_width = None
+        p.size_height = None
+        return
+
+    dimensions = parse_number(dimensions, first=False)
+    if len(dimensions) >= 3:
+        p.size_length = str(dimensions[0]) + " in"
+        p.size_width = str(dimensions[1]) + " in"
+        p.size_height = str(dimensions[2]) + " in"
+    elif len(dimensions) == 2:
+        p.size_length = str(dimensions[0]) + " in"
+        p.size_width = str(dimensions[1]) + " in"
+        p.size_height = None
+    elif len(dimensions) == 1:
+        p.size_length = str(dimensions[0]) + " in"
+        p.size_width = None
+        p.size_height = None
+    else:
+        p.size_length = None
+        p.size_width = None
+        p.size_height = None
+
+
+def parse_color(p: Product, selector: parsel.Selector = None):
+    color = selector.xpath("normalize-space(string(//div[@id='prodDetails']/span))").re_first("Color:(.+?)(?: |$)")
+    if color is None:
+        p.color = None
+    else:
+        p.color = color.lower()
+
+
+def parse_variation(p: Product, selector: parsel.Selector = None):
+    p.variation_values = None
+    p.selected_variations = None
+    p.asin_variation_values = None
